package org.sat4j.minisat.constraints;

import java.math.BigInteger;

import org.sat4j.core.Vec;
import org.sat4j.core.VecInt;
import org.sat4j.minisat.constraints.cnf.WLClause;
import org.sat4j.minisat.constraints.pb.IDataStructurePB;
import org.sat4j.minisat.constraints.pb.PBConstr;
import org.sat4j.minisat.constraints.pb.WatchPb;
import org.sat4j.specs.ContradictionException;
import org.sat4j.specs.IVec;
import org.sat4j.specs.IVecInt;

public abstract class AbstractPBClauseCardConstrDataStructure extends
		AbstractPBDataStructureFactory {

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.sat4j.minisat.constraints.AbstractPBDataStructureFactory#constraintFactory(org.sat4j.specs.VecInt,
	 *      org.sat4j.specs.VecInt, boolean, int)
	 */
	@Override
	protected PBConstr constraintFactory(IVecInt literals, IVecInt coefs,
			boolean moreThan, int degree) throws ContradictionException {
		return constraintFactory(literals, WatchPb.toVecBigInt(coefs),
				moreThan, WatchPb.toBigInt(degree));
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.sat4j.minisat.constraints.AbstractPBDataStructureFactory#constraintFactory(org.sat4j.specs.VecInt,
	 *      org.sat4j.specs.VecInt, int)
	 */
	@Override
	protected PBConstr constraintFactory(IVecInt literals, IVecInt coefs,
			int degree) {
		return constraintFactory(literals, WatchPb.toVecBigInt(coefs), WatchPb
				.toBigInt(degree));
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.sat4j.minisat.constraints.AbstractPBDataStructureFactory#constraintFactory(org.sat4j.specs.VecInt,
	 *      org.sat4j.specs.VecInt, boolean, int)
	 */
	@Override
	protected PBConstr constraintFactory(IVecInt literals,
			IVec<BigInteger> coefs, boolean moreThan, BigInteger degree)
			throws ContradictionException {
		IDataStructurePB mpb = WatchPb.niceParameters(literals, coefs,
				moreThan, degree, getVocabulary());
		if (mpb == null)
			return null;
		int size = mpb.size();
		int[] lits = new int[size];
		BigInteger[] normCoefs = new BigInteger[size];
		mpb.buildConstraintFromMapPb(lits, normCoefs);
		if (mpb.getDegree().equals(BigInteger.ONE)) {
			IVecInt v = WLClause.sanityCheck(new VecInt(lits), getVocabulary(),
					solver);
			if (v == null)
				return null;
			return constructClause(v);
		}
		if (coefficientsEqualToOne(new Vec<BigInteger>(normCoefs))) {
			assert mpb.getDegree().compareTo(new BigInteger(String.valueOf(Integer.MAX_VALUE))) < 0;
			return constructCard(new VecInt(lits), mpb.getDegree().intValue());
		}
		return constructPB(mpb);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.sat4j.minisat.constraints.AbstractPBDataStructureFactory#constraintFactory(org.sat4j.specs.VecInt,
	 *      org.sat4j.specs.VecInt, int)
	 */
	@Override
	protected PBConstr constraintFactory(IVecInt literals,
			IVec<BigInteger> coefs, BigInteger degree) {
		if (degree.equals(BigInteger.ONE)) {
			return constructLearntClause(literals);
		}
		if (coefficientsEqualToOne(coefs)) {
			return constructLearntCard(literals, degree.intValue());
		}
		return constructLearntPB(literals, coefs, degree);
	}

	private static boolean coefficientsEqualToOne(IVec<BigInteger> coeffs) {
		for (int i = 0; i < coeffs.size(); i++)
			if (!coeffs.get(i).equals(BigInteger.ONE))
				return false;
		return true;
	}

	abstract protected PBConstr constructClause(IVecInt v);

	abstract protected PBConstr constructCard(IVecInt lits, int degree)
			throws ContradictionException;

	abstract protected PBConstr constructPB(IDataStructurePB mpb)
			throws ContradictionException;

	abstract protected PBConstr constructLearntClause(IVecInt literals);

	abstract protected PBConstr constructLearntCard(IVecInt literals, int degree);

	abstract protected PBConstr constructLearntPB(IVecInt literals,
			IVec<BigInteger> coefs, BigInteger degree);

}
