/*******************************************************************************
 * Copyright (C) 2008 United States Government as represented by the
 * Administrator of the National Aeronautics and Space Administration
 * (NASA).  All Rights Reserved.
 * 
 * This software is distributed under the NASA Open Source Agreement
 * (NOSA), version 1.3.  The NOSA has been approved by the Open Source
 * Initiative.  See the file NOSA-1.3-JPF at the top of the distribution
 * directory tree for the complete NOSA document.
 * 
 * THE SUBJECT SOFTWARE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY OF ANY
 * KIND, EITHER EXPRESSED, IMPLIED, OR STATUTORY, INCLUDING, BUT NOT
 * LIMITED TO, ANY WARRANTY THAT THE SUBJECT SOFTWARE WILL CONFORM TO
 * SPECIFICATIONS, ANY IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
 * A PARTICULAR PURPOSE, OR FREEDOM FROM INFRINGEMENT, ANY WARRANTY THAT
 * THE SUBJECT SOFTWARE WILL BE ERROR FREE, OR ANY WARRANTY THAT
 * DOCUMENTATION, IF PROVIDED, WILL CONFORM TO THE SUBJECT SOFTWARE.
 ******************************************************************************/
package jfuzz.unitTests;

import java.io.File;

import jfuzz.JFuzz;
import jfuzz.termination.UpToFixedNumber;
import junit.framework.TestCase;

public abstract class AbstractSat4jTest extends TestCase {
    
    protected final File generated = new File("extensions/concolic/generated");
    
    /**
     * Directory that is checked for the processed files.
     */
    protected final File processed = new File(generated, "processed");
    
    /**
     * Directory that is checked for the queue files.
     */
    protected final File queued = new File(generated, "queue");
    
    /**
     * Directory that is checked for the error files.
     */
    protected final File error = new File(generated, "errors");
    
    /**
     * Helper to check if a file is a directory
     */
    protected void checkExistIsDir(File f) {
        assertTrue(f.getName() + " either doesn't exist or is not a directory!",
                f.exists() && f.isDirectory());
    }
    
    /**
     * 
     * @param inputFile = file to read or filename to pass to the class
     * @param symMethod = the method that is Symbolic with args separated by a #.  ex. foo(sym#sym)
     * @param className = the class to execute
     * @param numExec = the number of rounds of execution
     * @param retainData 
     * @return Arguments for JFuzz
     */
    protected String[] genArgs(String inputFile, String symMethod, String className, int numExec, boolean retainData) {
        return new String[] {"+" + JFuzz.JFUZZ_INPUT_PROP + "=" + inputFile, 
                "+jpf.basedir=../javapathfinder-trunk",
                "+jfuzz.input.cp=./lib/sat4j-apache.jar:.",
                "+vm.insn_factory.class=gov.nasa.jpf.concolic.ConcolicInstructionFactory",
                "+jpf.listener=jfuzz.ConcolicListener",
                "+symbolic.method=" + symMethod,
                "+search.class=gov.nasa.jpf.search.Simulation",
                "+jpf.report.console.finished=",
                "+" + JFuzz.JFUZZ_TERMINATION + "=jfuzz.termination.UpToFixedNumber",
                "+" + UpToFixedNumber.JFUZZ_NUM_EXEC + "=" + numExec,
                "+" + JFuzz.JFUZZ_CACHE + "=jfuzz.cache.NameIndependentCache",
                "+" + JFuzz.JFUZZ_DEL + "=" + retainData,
                className };
    }
    
    /**
     * 
     * @param inputFile = file to read or filename to pass to the class
     * @param symMethod = the method that is Symbolic with args separated by a #.  ex. foo(sym#sym)
     * @param className = the class to execute
     * @return Arguments for JFuzz
     */
    protected String[] genArgs(String inputFile, String symMethod, String className) {
        return genArgs(inputFile, symMethod, className, -1, false);
    }
    
    /**
     * 
     * @param inputFile = file to read or filename to pass to the class
     * @param symMethod = the method that is Symbolic with args separated by a #.  ex. foo(sym#sym)
     * @param className = the class to execute
     * @param retainData = true if inputs that don't produce errors should have their data saved
     * @return Arguments for JFuzz
     */
    protected String[] genArgs(String inputFile, String symMethod, String className, boolean retainData) {
        return genArgs(inputFile, symMethod, className, -1, retainData);
    }
}
