/*
 * Created on 29 mars 2006
 *
 * To change the template for this generated file go to
 * Window>Preferences>Java>Code Generation>Code and Comments
 */
package org.sat4j;


import java.io.PrintWriter;

import org.sat4j.specs.ContradictionException;
import org.sat4j.specs.IOptimizationProblem;
import org.sat4j.specs.IProblem;
import org.sat4j.specs.TimeoutException;

/**
 * This class is intended to be used by launchers to solve optimization problems,
 * i.e. problems for which a loop is needed to find the optimal solution.
 * 
 * @author leberre
 *
 */
@SuppressWarnings("PMD")
public abstract class AbstractOptimizationLauncher extends AbstractLauncher {

    /**
     * 
     */
    private static final long serialVersionUID = 1L;
    private static final String CURRENT_OPTIMUM_VALUE_PREFIX = "o "; //$NON-NLS-1$
 
    @Override
    protected void displayResult() {
        if (solver == null)
            return;
        PrintWriter out = getLogWriter();
        solver.printStat(out, COMMENT_PREFIX);
        ExitCode exitCode = getExitCode();
        out.println(ANSWER_PREFIX + exitCode);
        if (exitCode == ExitCode.SATISFIABLE
                || exitCode == ExitCode.OPTIMUM_FOUND) {
            out.println(SOLUTION_PREFIX + getReader().decode(solver.model()));
            IOptimizationProblem optproblem = (IOptimizationProblem)solver;
            if (!optproblem.hasNoObjectiveFunction()) {
                log("objective function=" + optproblem.calculateObjective()); //$NON-NLS-1$
            }
    
        }
        log("Total wall clock time (ms): " //$NON-NLS-1$
                + (System.currentTimeMillis() - getBeginTime()) / 1000.0);
    }

    @Override
    protected void solve(IProblem problem) throws TimeoutException {
            boolean isSatisfiable = false;
            
            IOptimizationProblem optproblem = (IOptimizationProblem) problem;  
            
            try {
                while (optproblem.admitABetterSolution()) {
                    if (!isSatisfiable) {
                        if (optproblem.nonOptimalMeansSatisfiable()) {
                            setExitCode(ExitCode.SATISFIABLE);
                            if (optproblem.hasNoObjectiveFunction()) {
                                return;
                            }
                            log("SATISFIABLE"); //$NON-NLS-1$
                        }
                        isSatisfiable = true;                        
                        log("OPTIMIZING..."); //$NON-NLS-1$
                    }
                    log("Got one! Elapsed wall clock time (in seconds):" //$NON-NLS-1$
                            + (System.currentTimeMillis() - getBeginTime())
                            / 1000.0);
                    getLogWriter().println(CURRENT_OPTIMUM_VALUE_PREFIX
                            + optproblem.calculateObjective());
                    optproblem.discard();
                }
                if (isSatisfiable) {
                    setExitCode(ExitCode.OPTIMUM_FOUND);
                } else {
                    setExitCode(ExitCode.UNSATISFIABLE);
                }
            } catch (ContradictionException ex) {
                assert isSatisfiable;
                setExitCode(ExitCode.OPTIMUM_FOUND);
            }
        }

}
