package org.sat4j;

import static java.lang.System.out;

import org.sat4j.minisat.SolverFactory;
import org.sat4j.reader.CSPExtSupportReader;
import org.sat4j.reader.Reader;
import org.sat4j.specs.ISolver;

public class CSPLauncher extends AbstractLauncher {

	/**
     * 
     */
    private static final long serialVersionUID = 1L;

    /* (non-Javadoc)
	 * @see org.sat4j.Lanceur#configureSolver(java.lang.String[])
	 */
	@Override
	protected ISolver configureSolver(String[] args) {	
		return SolverFactory.newMiniSAT2Heap();
	}

	/* (non-Javadoc)
	 * @see org.sat4j.Lanceur#createReader(org.sat4j.specs.ISolver)
	 */
	@Override
	protected Reader createReader(ISolver solver,String problemname) {
		return new CSPExtSupportReader(solver);
	}

	/* (non-Javadoc)
	 * @see org.sat4j.Lanceur#displayResult(org.sat4j.specs.ISolver, long, org.sat4j.ExitCode)
	 */
	@Override
	protected void displayResult() {
		switch (getExitCode()) {
		case SATISFIABLE:
			out.println("SAT"); //$NON-NLS-1$
			int[] model = solver.model();
            out.println(getReader().decode(model));
			break;
		case UNSATISFIABLE:
			out.println("UNSAT"); //$NON-NLS-1$
			break;
		default:
			break;
		}
		out.println(COMMENT_PREFIX+" Total ellapsed time (wall clock):"+(System.currentTimeMillis()-getBeginTime())/1000.0); //$NON-NLS-1$
	}

    public static void main(String[] args) {
        AbstractLauncher lanceur = new CSPLauncher();
        lanceur.run(args);
    }

	@Override
	protected void usage() {
		out.println("java -jar sat4jCSP instance-name-without-suffix"); //$NON-NLS-1$
	}

	@Override
	protected String getInstanceName(String[] args) {
		assert args.length==1;
		return args[0]=args[0]+".txt"; //$NON-NLS-1$
	}

}
