package org.sat4j;

/**
 * Enumeration allowing to manage easily exit code for the SAT and PB
 * Competitions.
 * 
 * @author leberre
 * 
 */
public enum ExitCode {
    OPTIMUM_FOUND(30, "OPTIMUM FOUND"), SATISFIABLE(10), UNKNOWN(0), UNSATISFIABLE( //$NON-NLS-1$
            20);

    /** value of the exit code. */
    private final int value;

    /** alternative textual representation of the exit code. */
    private final String str;

    /**
     * creates an exit code with a given value.
     * 
     * @param i
     *            the value of the exit code
     */
    ExitCode(final int i) {
        this.value = i;
        str = null;
    }

    /**
     * creates an exit code with a given value and an alternative textual
     * representation.
     * 
     * @param i
     *            the value of the exit code
     * @param str
     *            the alternative textual representation
     */
    ExitCode(final int i, final String str) {
        this.value = i;
        this.str = str;
    }

    /**
     * @return the exit code value
     */
    public int value() {
        return value;
    }

    /**
     * @return the name of the enum or the alternative textual
     *         representation if any.
     */
    @Override
    public String toString() {
        if (str != null) {
            return str;
        }
        return super.toString();
    }
}