/*
 * Created on Jan 20, 2005
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package org.sat4j.minisat.constraints.pb;

import java.math.BigInteger;

import org.sat4j.minisat.constraints.cnf.Lits;
import org.sat4j.minisat.core.ILits;
import org.sat4j.specs.IVec;
import org.sat4j.specs.IVecInt;

/**
 * @author parrain TODO To change the template for this generated type comment
 *         go to Window - Preferences - Java - Code Style - Code Templates
 */
public class ArrayPb implements IDataStructurePB {

	/*
	 * During the process of cutting planes, pseudo-boolean constraints are
	 * coded with a HashMap <literal, coefficient> and a BigInteger for the
	 * degree.
	 */
	protected final ILits voc;

	protected BigInteger[] coefs;

	protected BigInteger degree;


	ArrayPb(int[] lits, BigInteger[] coefLits, BigInteger degree, ILits voc) {
		coefs = new BigInteger[2 * voc.nVars() + 2];
		for (int i = 0; i < lits.length; i++)
			coefs[lits[i]] = coefLits[i];
		this.degree = degree;
		this.voc = voc;
	}

	ArrayPb(ILits voc) {
		coefs = new BigInteger[2 * voc.nVars() + 2];
		degree = BigInteger.ZERO;
		this.voc = voc;
	}

	public BigInteger saturation() {
		assert degree.signum() > 0;
		BigInteger minimum = degree;
		for (int i = 2; i < coefs.length; i++) {
			if (coefs[i] != null) {
				assert coefs[i].signum() > 0;
				coefs[i] = degree.min(coefs[i]);
				assert coefs[i].signum() > 0;
				minimum = minimum.min(coefs[i]);
			}
		}
		// on a appris une clause
		if (minimum.equals(degree) && minimum.compareTo(BigInteger.ONE) > 0) {
			degree = BigInteger.ONE;
			for (int i = 2; i < coefs.length; i++)
				if (coefs[i] != null)
					coefs[i] = BigInteger.ONE;
		}

		return degree;
	}

	public BigInteger cuttingPlane(PBConstr cpb, BigInteger deg,
			BigInteger[] reducedCoefs) {
		return cuttingPlane(cpb, deg, reducedCoefs, BigInteger.ONE);
	}

	public BigInteger cuttingPlane(PBConstr cpb, BigInteger degreeCons,
			BigInteger[] reducedCoefs, BigInteger coefMult) {
		degree = degree.add(degreeCons);
		assert degree.signum() > 0;

		if (reducedCoefs == null) {
			for (int i = 0; i < cpb.size(); i++)
				cuttingPlane(cpb.get(i), multiplyCoefficient(cpb.getCoef(i),
						coefMult));
		} else
			for (int i = 0; i < cpb.size(); i++)
				cuttingPlane(cpb.get(i), multiplyCoefficient(reducedCoefs[i],
						coefMult));

		return degree;
	}

	public BigInteger cuttingPlane(int[] lits, BigInteger[] reducedCoefs,
			BigInteger deg) {
		return cuttingPlane(lits, reducedCoefs, deg, BigInteger.ONE);
	}

	public BigInteger cuttingPlane(int lits[], BigInteger[] reducedCoefs,
			BigInteger degreeCons, BigInteger coefMult) {
		degree = degree.add(degreeCons);
		assert degree.signum() > 0;

		for (int i = 0; i < lits.length; i++)
			cuttingPlane(lits[i], reducedCoefs[i].multiply(coefMult));

		return degree;
	}

	private void cuttingPlane(final int lit, final BigInteger coef) {
		assert coef.signum() >= 0;
		if (coef.signum() > 0) {
			if (coefs[lit ^ 1] == null) {
				assert (coefs[lit] == null) || (coefs[lit].signum() > 0);
				if (coefs[lit] == null)
					coefs[lit] = coef;
				else
					coefs[lit] = coefs[lit].add(coef);
				assert coefs[lit].signum() > 0;
			} else {				
				assert coefs[lit] == null;
				if (coefs[lit ^ 1].compareTo(coef) < 0) {
					coefs[lit] = coef.subtract(coefs[lit ^ 1]);
					assert coefs[lit].signum() > 0;
					degree = degree.subtract(coefs[lit ^ 1]);
					coefs[lit ^ 1] = null;
				} else {
					if (coefs[lit ^ 1].equals(coef)) {
						degree = degree.subtract(coef);
						coefs[lit ^ 1] = null;
					} else {
						coefs[lit ^ 1] = coefs[lit ^ 1].subtract(coef);
						assert coefs[lit ^ 1].signum() > 0;
						degree = degree.subtract(coef);
					}
				}
			}
		}
		assert (coefs[lit ^ 1] == null) || (coefs[lit] == null);
	}

	public void buildConstraintFromConflict(IVecInt resLits,
			IVec<BigInteger> resCoefs) {
		resLits.clear();
		resCoefs.clear();
		// On recherche tous les litt?raux concern?s
		for (int i = 2; i < coefs.length; i++) {
			if (coefs[i] != null) {
				resLits.push(i);
				assert coefs[i].signum() > 0;
				resCoefs.push(coefs[i]);
			}
		}
	};

	public void buildConstraintFromMapPb(int[] resLits, BigInteger[] resCoefs) {
		// On recherche tous les litt?raux concern?s
		assert resLits.length == resCoefs.length;
		assert resLits.length == size();
		int ind = 0;
		for (int i = 2; i < coefs.length; i++) {
			if (coefs[i] != null) {
				resLits[ind] = i;
				assert coefs[i].signum() > 0;
				resCoefs[ind] = coefs[i];
				ind++;
			}
		}
	};

	public BigInteger getDegree() {
		return degree;
	}

	public int size() {
		int cpt = 0;
		for (int i = 2; i < coefs.length; i++)
			if (coefs[i] != null)
				cpt++;
		return cpt;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		StringBuilder stb = new StringBuilder();
		for (int i = 2; i < coefs.length; i++) {
			if (coefs[i] != null) {
				stb.append(coefs[i]);
				stb.append(".");
				stb.append(Lits.toString(i));
				stb.append(" ");
				stb.append("[");
				stb.append(voc.valueToString(i));
				stb.append("]");

			}
		}
		return stb.toString() + " >= " + degree; //$NON-NLS-1$
	}

	private BigInteger multiplyCoefficient(BigInteger coef, BigInteger mult) {
		if (mult.equals(BigInteger.ONE))
			return coef;
		if (coef.equals(BigInteger.ONE))
			return mult;
		return coef.multiply(mult);
	}

}
