package org.sat4j.minisat.constraints.pb;

import java.math.BigInteger;

import org.sat4j.minisat.constraints.card.AtLeast;
import org.sat4j.minisat.core.Constr;
import org.sat4j.minisat.core.ILits;
import org.sat4j.minisat.core.UnitPropagationListener;
import org.sat4j.specs.ContradictionException;
import org.sat4j.specs.IVecInt;

public class AtLeastPB extends AtLeast implements PBConstr {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	
	private final BigInteger degree;

	private AtLeastPB(ILits voc, IVecInt ps, int degree) {
		super(voc, ps, degree);
		this.degree = new BigInteger(String.valueOf(degree));
	}

	public static AtLeastPB atLeastNew(UnitPropagationListener s, ILits voc,
			IVecInt ps, int n) throws ContradictionException {
		int degree = niceParameters(s, voc, ps, n);
		if (degree == 0)
			return null;
		return new AtLeastPB(voc, ps, degree);
	}

	public static AtLeastPB atLeastNew(ILits voc, IVecInt ps, int n) {
		return new AtLeastPB(voc, ps, n);
	}

	public BigInteger getCoef(int literal) {
		return BigInteger.ONE;
	}

	public BigInteger getDegree() {
		return degree;
	}

	public ILits getVocabulary() {
		return voc;
	}

	public int[] getLits() {
		int[] tmp = new int[size()];
		System.arraycopy(lits, 0, tmp, 0, size());
		return tmp;
	}

	public BigInteger[] getCoefs() {
		BigInteger[] tmp = new BigInteger[size()];
		for (int i = 0; i < tmp.length; i++)
			tmp[i] = BigInteger.ONE;
		return tmp;
	}

	/**
	 * 
	 */
	private boolean learnt = false;

	/**
	 * D?termine si la contrainte est apprise
	 * 
	 * @return true si la contrainte est apprise, false sinon
	 * @see Constr#learnt()
	 */
	public boolean learnt() {
		return learnt;
	}

	public void setLearnt() {
		learnt = true;
	}

	public void register() {
		assert learnt;
		//countFalsified();
	}

	public void assertConstraint(UnitPropagationListener s) {
		for (int i = 0; i < size(); i++) {
			if (getVocabulary().isUnassigned(get(i))) {
				boolean ret = s.enqueue(get(i), this);
				assert ret;
			}
		}
	}

	public IVecInt computeAnImpliedClause() {
		return null;
	}


}
