/*
 * Created on Jan 17, 2005
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package org.sat4j.minisat.constraints.pb;

import java.math.BigInteger;

import org.sat4j.minisat.constraints.cnf.Lits;
import org.sat4j.minisat.core.ILits;

/**
 * @author parrain TODO To change the template for this generated type comment
 *         go to Window - Preferences - Java - Code Style - Code Templates
 */
class ConflictArray extends ArrayPb implements IConflict {

	public static IConflict createConflict(PBConstr cpb) {
		return new ConflictArray(cpb.getLits(), cpb.getCoefs(),
				cpb.getDegree(), cpb.getVocabulary());
	}

	ConflictArray(int[] lits, BigInteger[] coefs, BigInteger d, ILits voc) {
		super(lits, coefs, d, voc);
	}

	/*
	 * coefficient to be computed.
	 * 
	 */
	protected BigInteger coefMult = BigInteger.ZERO;

	protected BigInteger coefMultCons = BigInteger.ZERO;

	/**
	 * Effectue une resolution avec une contrainte PB. Met a jour le Conflict.
	 * 
	 * @param cpb
	 *            contrainte avec laquelle on va faire la resolution
	 * @param litImplied
	 *            litteral devant etre resolu
	 * @return la mise a jour du degre
	 */
	public BigInteger resolve(PBConstr cpb, int litImplied) {
		assert litImplied > 1;

		if (coefs[litImplied ^ 1] == null) {
			// logger.fine("pas de resolution");
			return degree;
		}

		assert slackConflict().signum() <= 0;
		assert degree.signum() >= 0;

		// copie des coeffs de la contrainte pour pouvoir effectuer
		// des operations de reductions
		BigInteger[] coefsCons = null;
		BigInteger degreeCons = cpb.getDegree();

		// Recherche de l'indice du litteral implique
		int ind = 0;
		while (cpb.get(ind) != litImplied)
			ind++;

		assert cpb.get(ind) == litImplied;
		assert cpb.getCoef(ind) != BigInteger.ZERO;

		if (cpb.getCoef(ind).equals(BigInteger.ONE)) {
			// la rsolvante sera conflictuelle (Dixon)
			coefMultCons = coefs[litImplied ^ 1];
			coefMult = BigInteger.ONE;
			// mise-a-jour du degre du conflit
			degreeCons = (degreeCons.multiply(coefMultCons));
		} else {
			if (coefs[litImplied ^ 1].equals(BigInteger.ONE)) {
				// la rsolvante sera conflictuelle (Dixon)
				coefMult = cpb.getCoef(ind);
				coefMultCons = BigInteger.ONE;
				// mise-a-jour du degre du conflit
				degree = degree.multiply(coefMult);
			} else {
				// Reduction de la contrainte
				// jusqu'a obtenir une resolvante conflictuelle
				WatchPb wpb = (WatchPb) cpb;
				coefsCons = wpb.getCoefs();
				assert positiveCoefs(coefsCons);
				degreeCons = reduceUntilConflict(litImplied, ind, coefsCons,
						wpb);
				// mise-a-jour du degre du conflit
				degreeCons = degreeCons.multiply(coefMultCons);
				degree = degree.multiply(coefMult);
			}
			// multiplication par coefMult des coefficients du conflit
			if (!coefMult.equals(BigInteger.ONE))
				for (int i = 2; i < coefs.length; i++) {
					if (coefs[i] != null)
						coefs[i] = coefs[i].multiply(coefMult);
				}
		}

		// On effectue la resolution
		degree = cuttingPlane(cpb, degreeCons, coefsCons, coefMultCons);
		assert coefs[litImplied] == null;
		assert coefs[litImplied ^ 1] == null;
		assert degree.signum() > 0;

		// Saturation
		degree = saturation();
		assert slackConflict().signum() <= 0;
		return degree;
	}

	/**
	 * reduces the constraint defined by wpb until the result of the cutting
	 * plane is a conflict. this reduction returns as much as poaaible a
	 * constraint which is still a pseudo-boolean constraint.
	 * 
	 * @param litImplied
	 * @param ind
	 * @param reducedCoefs
	 * @param wpb
	 * @return
	 */
	protected BigInteger reduceUntilConflict(int litImplied, int ind,
			BigInteger[] reducedCoefs, WatchPb wpb) {
		BigInteger slackResolve = BigInteger.ONE.negate();
		BigInteger slackThis = BigInteger.ZERO;
		BigInteger slackIndex = BigInteger.ZERO;
		BigInteger ppcm;
		BigInteger reducedDegree = wpb.getDegree();


		do {
			if (slackResolve.signum() >= 0) {
				assert slackThis.signum() > 0;
				BigInteger tmp = reduceInConstraint(wpb, reducedCoefs, ind,
						reducedDegree);
				assert ((tmp.compareTo(reducedDegree) < 0) && (tmp
						.compareTo(BigInteger.ONE) >= 0));
				reducedDegree = tmp;
			}
			// Recherche des coefficients multiplicateurs
			assert coefs[litImplied ^ 1].signum() > 0;
			assert reducedCoefs[ind].signum() > 0;

			ppcm = ppcm(reducedCoefs[ind], coefs[litImplied ^ 1]);
			assert ppcm.signum() > 0;
			coefMult = ppcm.divide(coefs[litImplied ^ 1]);
			coefMultCons = ppcm.divide(reducedCoefs[ind]);

			assert coefMultCons.signum() > 0;
			assert coefMult.signum() > 0;
			assert coefMult.multiply(coefs[litImplied ^ 1]).equals(
					coefMultCons.multiply(reducedCoefs[ind]));

			// calcul des marges (poss) de chaque contrainte
			slackThis = wpb.slackConstraint(reducedCoefs, reducedDegree)
					.multiply(coefMultCons);
			slackIndex = slackConflict().multiply(coefMult);

			assert slackIndex.signum() <= 0;

			// calcul (approximation) de la marge de la resolvante
			slackResolve = slackThis.add(slackIndex);
		} while (slackResolve.signum() >= 0);
		assert coefMult.multiply(coefs[litImplied ^ 1]).equals(
				coefMultCons.multiply(reducedCoefs[ind]));
		return reducedDegree;

	}

	public boolean affiche = false;

	public BigInteger slackConflict() {
		BigInteger poss = BigInteger.ZERO;
		// Pour chaque litteral
		for (int i = 2; i < coefs.length; i++)
			if (coefs[i] != null && coefs[i].signum() != 0
					&& !voc.isFalsified(i)) {
				poss = poss.add(coefs[i]);
				if (affiche)
					System.out.println("litteral non falsifie :"
							+ Lits.toString(i));
			}
		return poss.subtract(degree);
	}

	public boolean isAssertive(int dl) {
		BigInteger slack = BigInteger.ZERO;
		for (int i = 2; i < coefs.length; i++) {
			if (coefs[i] != null && (coefs[i].signum() > 0)
					&& (((!voc.isFalsified(i)) || voc.getLevel(i) >= dl)))
				slack = slack.add(coefs[i]);
		}
		slack = slack.subtract(degree);
		if (slack.signum() < 0)
			return false;
		for (int i = 2; i < coefs.length; i++) {
			if (coefs[i] != null && (coefs[i].signum() > 0)
					&& (voc.isUnassigned(i) || voc.getLevel(i) >= dl)
					&& (slack.subtract(coefs[i]).signum() < 0))
				return true;
		}
		return false;
	}

	/**
	 * Calcule le ppcm de deux nombres
	 * 
	 * @param a
	 *            premier nombre de l'op?ration
	 * @param b
	 *            second nombre de l'op?ration
	 * @return le ppcm en question
	 */
	protected static BigInteger ppcm(BigInteger a, BigInteger b) {
		return a.divide(a.gcd(b)).multiply(b);
	}

	/**
	 * Reduction d'une contrainte On supprime un litteral non assigne
	 * prioritairement, vrai sinon. En aucun cas on ne supprime litImplied.
	 * 
	 * 
	 * @return mise a jour du degre
	 */
	public BigInteger reduceInConstraint(WatchPb wpb,
			final BigInteger[] coefsBis, final int indLitImplied,
			final BigInteger degreeBis) {
		// logger.entering(this.getClass().getName(),"reduceInConstraint");
		assert degreeBis.compareTo(BigInteger.ONE) > 0;
		// Recherche d'un litt?ral non assign?
		int lit = -1;
		for (int ind = 0; (ind < wpb.size()) && (lit == -1); ind++)
			if (coefsBis[ind].signum() != 0 && voc.isUnassigned(wpb.get(ind))) {
				assert coefsBis[ind].compareTo(degreeBis) < 0;
				lit = ind;
			}

		// Sinon, recherche d'un litteral satisfait
		if (lit == -1)
			for (int ind = 0; (ind < wpb.size()) && (lit == -1); ind++)
				if ((coefsBis[ind].signum() != 0)
						&& (voc.isSatisfied(wpb.get(ind)))
						&& (ind != indLitImplied))
					lit = ind;

		// on a trouve un litteral
		assert lit != -1;

		assert lit != indLitImplied;
		// logger.finer("Found literal "+Lits.toString(lits[lit]));
		// on reduit la contrainte
		BigInteger degUpdate = degreeBis.subtract(coefsBis[lit]);
		coefsBis[lit] = BigInteger.ZERO;

		// saturation de la contrainte
		assert degUpdate.signum() > 0;
		BigInteger minimum = degUpdate;
		for (int i = 0; i < coefsBis.length; i++) {
			if (coefsBis[i].signum() > 0)
				minimum = minimum.min(coefsBis[i]);
			coefsBis[i] = degUpdate.min(coefsBis[i]);
		}
		if (minimum.equals(degUpdate) && !degUpdate.equals(BigInteger.ONE)) {
			// on a obtenu une clause
			// plus de reduction possible
			degUpdate = BigInteger.ONE;
			for (int i = 0; i < coefsBis.length; i++)
				if (coefsBis[i].signum() > 0)
					coefsBis[i] = degUpdate;
		}

		assert coefsBis[indLitImplied].signum() > 0;
		assert degreeBis.compareTo(degUpdate) > 0;
		return degUpdate;
	}

	private boolean positiveCoefs(final BigInteger[] coefsCons) {
		for (int i = 0; i < coefsCons.length; i++) {
			if (coefsCons[i].signum() <= 0)
				return false;
		}
		return true;
	}

	/**
	 * retourne le niveau de backtrack : c'est-?-dire le niveau le plus haut
	 * pour lequel la contrainte est assertive
	 * 
	 * @param maxLevel
	 *            le plus bas niveau pour lequel la contrainte est assertive
	 * @return the highest level (smaller int) for which the constraint is
	 *         assertive.
	 */
	public int getBacktrackLevel(int maxLevel) {
		int litLevel;
		int borneMax = maxLevel;
		// System.out.println(this);
		// System.out.println("assertive au niveau : " + maxLevel);
		assert isAssertive(borneMax);
		int borneMin = -1;
		// on calcule borneMax,
		// le niveau le plus haut dans l'arbre ou la contrainte est assertive
		for (int lit = 2; lit < coefs.length; lit++) {
			if (coefs[lit] != null) {
				litLevel = voc.getLevel(lit);
				if (litLevel < borneMax && litLevel > borneMin)
					if (isAssertive(litLevel))
						borneMax = litLevel;
					else
						borneMin = litLevel;
			}
		}
		// on retourne le niveau immediatement inferieur ? borneMax
		// pour lequel la contrainte possede un literal
		int retour = 0;
		for (int lit = 2; lit < coefs.length; lit++) {
			if (coefs[lit] != null) {
				litLevel = voc.getLevel(lit);
				if (litLevel > retour && litLevel < borneMax)
					retour = litLevel;
			}
		}
		return retour;
	}

}
