package org.sat4j.minisat.constraints.pb;

/*
 * Created on Jan 17, 2005
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */


import java.math.BigInteger;
import java.util.HashMap;
import java.util.Map;

import org.sat4j.minisat.core.ILits;

/**
 * @author parrain TODO To change the template for this generated type comment
 *         go to Window - Preferences - Java - Code Style - Code Templates
 */
public class ConflictMap extends MapPb implements IConflict{

	private final ILits voc;

    public static IConflict createConflict(PBConstr cpb) {
        Map<Integer, BigInteger> m = new HashMap<Integer, BigInteger>();
        for (int i = 0; i < cpb.size(); i++) {
            assert cpb.get(i) != 0;
            assert cpb.getCoef(i).signum() > 0;
            m.put(cpb.get(i), cpb.getCoef(i));
        }
        return new ConflictMap(m, cpb.getDegree(), cpb.getVocabulary());
    }


	ConflictMap(Map<Integer, BigInteger> m, BigInteger d, ILits voc) {
		super(m, d);
		this.voc = voc;
	}

	/*
	 * coefficient to be computed.
	 * 
	 */
	private BigInteger coefMult = BigInteger.ZERO;

	private BigInteger coefMultCons = BigInteger.ZERO;

	/**
	 * Effectue une resolution avec une contrainte PB. Met a jour le Conflict.
	 * 
	 * @param cpb
	 *            contrainte avec laquelle on va faire la resolution
	 * @param litImplied
	 *            litteral devant etre resolu
	 * @return la mise a jour du degre
	 */
	public BigInteger resolve(PBConstr cpb, int litImplied) {
		assert litImplied > 1;
		if (!coefs.containsKey(litImplied ^ 1)) {
			// logger.fine("pas de resolution");
			return degree;
		}

		assert slackConflict().signum() <= 0;
		assert degree.signum() >= 0;

		// copie des coeffs de la contrainte pour pouvoir effectuer
		// des operations de reductions
		BigInteger[] coefsCons = null;
		BigInteger degreeCons = cpb.getDegree();

		// Recherche de l'indice du litteral implique
		int ind = 0;
		while (cpb.get(ind) != litImplied)
			ind++;

		assert cpb.get(ind) == litImplied;
		assert cpb.getCoef(ind) != BigInteger.ZERO;

		if (cpb.getCoef(ind).equals(BigInteger.ONE)) {
			// la rsolvante sera conflictuelle (Dixon)
			coefMultCons = coefs.get(litImplied ^ 1);
			coefMult = BigInteger.ONE;
			// mise-a-jour du degre du conflit
			degreeCons = (degreeCons.multiply(coefMultCons));
		} else {
			if (coefs.get(litImplied ^ 1).equals(BigInteger.ONE)) {
				// la rsolvante sera conflictuelle (Dixon)
				coefMult = cpb.getCoef(ind);
				coefMultCons = BigInteger.ONE;
				// mise-a-jour du degre du conflit
				degree = degree.multiply(coefMult);
			} else {
				// Reduction de la contrainte
				// jusqu'a obtenir une resolvante conflictuelle
				WatchPb wpb = (WatchPb) cpb;
				coefsCons = wpb.getCoefs();
				assert positiveCoefs(coefsCons);
				degreeCons = reduceUntilConflict(litImplied, ind, coefsCons,
						wpb);
				// mise-a-jour du degre du conflit
				degreeCons = (degreeCons.multiply(coefMultCons));
				degree = degree.multiply(coefMult);
			}
			// multiplication par coefMult des coefficients du conflit
			for (Integer i : coefs.keySet()) {
				coefs.put(i, coefs.get(i).multiply(coefMult));
			}
		}

		// On effectue la resolution
		degree = cuttingPlane(cpb, degreeCons, coefsCons, coefMultCons);
		assert !coefs.containsKey(litImplied);
		assert !coefs.containsKey(litImplied ^ 1);
		assert degree.signum() > 0;

		// Saturation
		degree = saturation();
		assert slackConflict().signum() <= 0;

		return degree;
	}

	private BigInteger reduceUntilConflict(int litImplied, int ind,
			BigInteger[] reducedCoefs, WatchPb wpb) {
		BigInteger slackResolve = BigInteger.ONE.negate();
		BigInteger slackThis = BigInteger.ZERO;
		BigInteger slackIndex = BigInteger.ZERO;
		BigInteger ppcm;
		BigInteger reducedDegree = wpb.getDegree();

		do {
			if (slackResolve.signum() >= 0) {
				assert slackThis.signum() > 0;
				BigInteger tmp = reduceInConstraint(wpb, reducedCoefs, ind,
						reducedDegree);
				assert ((tmp.compareTo(reducedDegree) < 0) && (tmp
						.compareTo(BigInteger.ONE) >= 0));
				reducedDegree = tmp;
			}
			// Recherche des coefficients multiplicateurs
			assert coefs.get(litImplied ^ 1).signum() > 0;
			assert reducedCoefs[ind].signum() > 0;
			
			BigInteger coefLitImplied =  coefs.get(litImplied ^ 1);
			ppcm = ppcm(reducedCoefs[ind], coefLitImplied);
			assert ppcm.signum() > 0;
			coefMult = ppcm.divide(coefLitImplied);
			coefMultCons = ppcm.divide(reducedCoefs[ind]);

			assert coefMultCons.signum() > 0;
			assert coefMult.signum() > 0;
			assert coefMult.multiply(coefLitImplied).equals(
					coefMultCons.multiply(reducedCoefs[ind]));

			// calcul des marges (poss) de chaque contrainte
			slackThis = wpb.slackConstraint(reducedCoefs, reducedDegree)
					.multiply(coefMultCons);
			slackIndex = slackConflict().multiply(coefMult);

			assert slackIndex.signum() <= 0;

			// calcul (approximation) de la marge de la resolvante
			slackResolve = slackThis.add(slackIndex);
		} while (slackResolve.signum() >= 0);
		assert coefMult.multiply(coefs.get(litImplied ^ 1)).equals(
				coefMultCons.multiply(reducedCoefs[ind]));
		return reducedDegree;

	}

	public BigInteger slackConflict() {
		BigInteger poss = BigInteger.ZERO;
		BigInteger tmp;
		// Pour chaque litteral
		for (Integer i : coefs.keySet()){
			tmp = coefs.get(i);
			if (tmp.signum() != 0 && !voc.isFalsified(i))
				poss = poss.add(tmp);
		}
		return poss.subtract(degree);
	}

	public boolean isAssertive(int dl) {
		BigInteger slack = BigInteger.ZERO;
		BigInteger tmp;
		for (Integer i : coefs.keySet()) {
			tmp = coefs.get(i);
			if ((tmp.signum() > 0)
					&& (((!voc.isFalsified(i)) || voc.getLevel(i) >= dl)))
				slack = slack.add(tmp);
		}
		slack = slack.subtract(degree);
		if (slack.signum() < 0)
			return false;
		for (Integer i : coefs.keySet()) {
			tmp = coefs.get(i);
			if ((tmp.signum() > 0)
					&& (voc.isUnassigned(i) || voc.getLevel(i) >= dl)
					&& (slack.subtract(tmp).signum() < 0))
				return true;
		}
		return false;
	}

	/**
	 * Calcule le ppcm de deux nombres
	 * 
	 * @param a
	 *            premier nombre de l'op?ration
	 * @param b
	 *            second nombre de l'op?ration
	 * @return le ppcm en question
	 */
	protected static BigInteger ppcm(BigInteger a, BigInteger b) {
		return a.divide(a.gcd(b)).multiply(b);
	}

	/**
	 * Reduction d'une contrainte On supprime un litteral non assigne
	 * prioritairement, vrai sinon. En aucun cas on ne supprime litImplied.
	 * 
	 * @return mise a jour du degre
	 */
	public BigInteger reduceInConstraint(WatchPb wpb,
			final BigInteger[] coefsBis, final int indLitImplied,
			final BigInteger degreeBis) {
		// logger.entering(this.getClass().getName(),"reduceInConstraint");
		assert degreeBis.compareTo(BigInteger.ONE) > 0;
		// Recherche d'un litt?ral non assign?
		int lit = -1;
		for (int ind = 0; (ind < wpb.size()) && (lit == -1); ind++)
			if (coefsBis[ind].signum() != 0 && voc.isUnassigned(wpb.get(ind))) {
				assert coefsBis[ind].compareTo(degreeBis) < 0;
				lit = ind;
			}

		// Sinon, recherche d'un litteral satisfait
		if (lit == -1)
			for (int ind = 0; (ind < wpb.size()) && (lit == -1); ind++)
				if ((coefsBis[ind].signum() != 0)
						&& (voc.isSatisfied(wpb.get(ind)))
						&& (ind != indLitImplied))
					lit = ind;

		// on a trouve un litteral
		assert lit != -1;

		assert lit != indLitImplied;
		// logger.finer("Found literal "+Lits.toString(lits[lit]));
		// on reduit la contrainte
		BigInteger degUpdate = degreeBis.subtract(coefsBis[lit]);
		coefsBis[lit] = BigInteger.ZERO;

		// saturation de la contrainte
		assert degUpdate.signum() > 0;
		BigInteger minimum = degUpdate;
		for (int i = 0; i < coefsBis.length; i++) {
			if (coefsBis[i].signum() > 0)
				minimum = minimum.min(coefsBis[i]);
			coefsBis[i] = degUpdate.min(coefsBis[i]);
		}
		if (minimum.equals(degUpdate) && !degUpdate.equals(BigInteger.ONE)) {
			// on a obtenu une clause
			// plus de reduction possible
			degUpdate = BigInteger.ONE;
			for (int i = 0; i < coefsBis.length; i++)
				if (coefsBis[i].signum() > 0)
					coefsBis[i] = degUpdate;
		}

		assert coefsBis[indLitImplied].signum() > 0;
		assert degreeBis.compareTo(degUpdate) > 0;
		return degUpdate;
	}

	private boolean positiveCoefs(final BigInteger[] coefsCons) {
		for (int i = 0; i < coefsCons.length; i++) {
			if (coefsCons[i].signum() <= 0)
				return false;
		}
		return true;
	}

    /**
     * retourne le niveau de backtrack : c'est-?-dire le niveau le plus haut
     * pour lequel la contrainte est assertive
     * 
     * @param maxLevel 
     *            le plus bas niveau pour lequel la contrainte est assertive
     * @return the highest level (smaller int) for which the constraint is assertive.
     */
    public int getBacktrackLevel(int maxLevel) {
        int litLevel;
        int borneMax = maxLevel;
        // System.out.println(this);
        // System.out.println("assertive au niveau : " + maxLevel);
        assert isAssertive(borneMax);
        int borneMin = -1;
        // on calcule borneMax,
        // le niveau le plus haut dans l'arbre ou la contrainte est assertive
        for (int lit : coefs.keySet()) {
            litLevel = voc.getLevel(lit);
            if (litLevel < borneMax && litLevel > borneMin)
                if (isAssertive(litLevel))
                    borneMax = litLevel;
                else
                    borneMin = litLevel;
        }
        // on retourne le niveau immediatement inferieur ? borneMax
        // pour lequel la contrainte possede un literal
        int retour = 0;
        for (int lit : coefs.keySet()) {
            litLevel = voc.getLevel(lit);
            if (litLevel > retour && litLevel < borneMax)
                retour = litLevel;
        }
        return retour;
    }

}


