/*
 * Created on Jan 20, 2005
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package org.sat4j.minisat.constraints.pb;

import java.math.BigInteger;
import java.util.HashMap;
import java.util.Map;

import org.sat4j.minisat.constraints.cnf.Lits;
import org.sat4j.specs.IVec;
import org.sat4j.specs.IVecInt;

/**
 * @author parrain TODO To change the template for this generated type comment
 *         go to Window - Preferences - Java - Code Style - Code Templates
 */
public class MapPb implements IDataStructurePB {

	/*
	 * During the process of cutting planes, pseudo-boolean constraints are
	 * coded with a HashMap <literal, coefficient> and a BigInteger for the
	 * degree.
	 */
	protected Map<Integer, BigInteger> coefs;

	protected BigInteger degree;

	MapPb(Map<Integer, BigInteger> m, BigInteger d) {
		coefs = m;
		degree = d;
	}

	MapPb() {
		this(new HashMap<Integer, BigInteger>(), BigInteger.ZERO);
	}

	public BigInteger saturation() {
		assert degree.signum() > 0;
		BigInteger minimum = degree;
		for (Map.Entry<Integer, BigInteger> e : coefs.entrySet()) {
			assert e.getValue().signum() > 0;
			e.setValue(degree.min(e.getValue()));
			assert e.getValue().signum() > 0;
			minimum = minimum.min(e.getValue());
		}
		// on a appris une clause
		if (minimum.equals(degree) && minimum.compareTo(BigInteger.ONE) > 0) {
			degree = BigInteger.ONE;
			for (Integer i : coefs.keySet())
				coefs.put(i, BigInteger.ONE);
		}

		return degree;
	}

	public BigInteger cuttingPlane(PBConstr cpb, BigInteger deg,
			BigInteger[] reducedCoefs) {
		return cuttingPlane(cpb, deg, reducedCoefs, BigInteger.ONE);
	}

	public BigInteger cuttingPlane(PBConstr cpb, BigInteger degreeCons,
			BigInteger[] reducedCoefs, BigInteger coefMult) {
		degree = degree.add(degreeCons);
		assert degree.signum() > 0;

		if (reducedCoefs == null)
			for (int i = 0; i < cpb.size(); i++)
				cuttingPlane(cpb.get(i), multiplyCoefficient(cpb.getCoef(i), coefMult));
		else
			for (int i = 0; i < cpb.size(); i++)
				cuttingPlane(cpb.get(i), multiplyCoefficient(reducedCoefs[i], coefMult));

		return degree;
	}

	public BigInteger cuttingPlane(int[] lits, BigInteger[] reducedCoefs,
			BigInteger deg) {
		return cuttingPlane(lits, reducedCoefs, deg, BigInteger.ONE);
	}

	public BigInteger cuttingPlane(int lits[], BigInteger[] reducedCoefs,
			BigInteger degreeCons, BigInteger coefMult) {
		degree = degree.add(degreeCons);
		assert degree.signum() > 0;

		for (int i = 0; i < lits.length; i++)
			cuttingPlane(lits[i], reducedCoefs[i].multiply(coefMult));

		return degree;
	}

	private void cuttingPlane(final int lit, final BigInteger coef) {
		assert coef.signum() >= 0;
		if (coef.signum() > 0) {
			if (coefs.containsKey(lit ^ 1)) {
				assert !coefs.containsKey(lit);
				if (coefs.get(lit ^ 1).compareTo(coef) < 0) {
					BigInteger tmp = coefs.get(lit ^ 1);
					coefs.put(lit, coef.subtract(tmp));
					assert coefs.get(lit).signum() > 0;
					degree = degree.subtract(tmp);
					coefs.remove(lit ^ 1);
				} else {
					if (coefs.get(lit ^ 1).equals(coef)) {
						degree = degree.subtract(coef);
						coefs.remove(lit ^ 1);
					} else {
						coefs.put(lit ^ 1, coefs.get(lit ^ 1).subtract(coef));
						assert coefs.get(lit ^ 1).signum() > 0;
						degree = degree.subtract(coef);
					}
				}
			} else {
				assert (!coefs.containsKey(lit))
						|| (coefs.get(lit).signum() > 0);
				coefs.put(lit, (coefs.containsKey(lit) ? coefs.get(lit)
						: BigInteger.ZERO).add(coef));
				assert coefs.get(lit).signum() > 0;
			}
		}
		assert (!coefs.containsKey(lit ^ 1)) || (!coefs.containsKey(lit));
	}

	public void buildConstraintFromConflict(IVecInt resLits,
			IVec<BigInteger> resCoefs) {
		resLits.clear();
		resCoefs.clear();
		// On recherche tous les litt?raux concern?s
		for (Map.Entry<Integer, BigInteger> e : coefs.entrySet()) {
			resLits.push(e.getKey());
			assert e.getValue().signum() > 0;
			resCoefs.push(e.getValue());
		}
	};

	public void buildConstraintFromMapPb(int[] resLits, BigInteger[] resCoefs) {
		// On recherche tous les litt?raux concern?s
		assert resLits.length == resCoefs.length;
		assert resLits.length == coefs.keySet().size();
		int i = 0;
		for (Map.Entry<Integer, BigInteger> e : coefs.entrySet()) {
			resLits[i] = e.getKey();
			assert e.getValue().signum() > 0;
			resCoefs[i] = e.getValue();
			i++;
		}
	};

	public BigInteger getDegree() {
		return degree;
	}

	public int size() {
		return coefs.keySet().size();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		StringBuilder stb = new StringBuilder();
		for (Map.Entry<Integer, BigInteger> entry : coefs.entrySet()) {
			stb.append(entry.getValue());
			stb.append(".");
			stb.append(Lits.toString(entry.getKey()));
			stb.append(" ");
		}
		return stb.toString() + " >= " + degree; //$NON-NLS-1$
	}

	private BigInteger multiplyCoefficient(BigInteger coef,
			BigInteger mult) {
		if (coef.equals(BigInteger.ONE))
			return mult;
		else
			return coef.multiply(mult);
	}


}
