package org.sat4j.minisat.core;

import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;
import java.util.Stack;

@SuppressWarnings("serial")
public class DotSearchListener implements SearchListener {

    private final transient Stack<String> pile;

    private transient String currentNodeName = null;

    private transient Writer out;

    private transient boolean estOrange = false;

    public DotSearchListener(final String fileNameToSave) {
        pile = new Stack<String>();
        try {
            out = new FileWriter(fileNameToSave);
        } catch (IOException e) {
            System.err.println("Problem when created file.");
        }
    }

    public final void assuming(final int p) {
        final Integer absP = new Integer(Math.abs(p));
        String newName;

        if (currentNodeName == null) {
            newName = absP.toString();
            pile.push(newName);
            saveLine(lineTab("\"" + newName + "\"" + "[label=\"" + newName
                    + "\", shape=circle, color=blue, style=filled]"));
        } else {
            newName = currentNodeName;
            pile.push(newName);
            saveLine(lineTab("\"" + newName + "\"" + "[label=\""
                    + absP.toString()
                    + "\", shape=circle, color=blue, style=filled]"));
        }
        currentNodeName = newName;
    }

    public final void propagating(final int p) {
        String newName = currentNodeName + "." + p;

        if (currentNodeName == null) {
            saveLine(lineTab("\"null\" [label=\"\", shape=point]"));
        }
        final String couleur = estOrange ? "orange" : "green";

        saveLine(lineTab("\"" + newName + "\"" + "[label=\""
                + new Integer(p).toString() + "\",shape=point, color=black]"));
        saveLine(lineTab("\"" + currentNodeName + "\"" + " -- " + "\""
                + newName + "\"" + "[label=" + "\" "
                + new Integer(p).toString() + "\", fontcolor =" + couleur
                + ", color = " + couleur + ", style = bold]"));
        currentNodeName = newName;
        estOrange = false;
    }

    public final void backtracking(final int p) {
        final String temp = pile.pop();

        saveLine("\"" + temp + "\"" + "--" + "\"" + currentNodeName + "\""
                + "[label=\"\", color=red, style=dotted]");
        currentNodeName = temp;
    }

    public final void adding(final int p) {
        estOrange = true;
    }

    public final void learn(final Constr clause) {
    }

    public final void delete(final int[] clause) {
    }

    public final void conflictFound() {
        saveLine(lineTab("\"" + currentNodeName
                + "\" [label=\"\", shape=box, color=\"red\", style=filled]"));
    }

    public final void solutionFound() {
        saveLine(lineTab("\"" + currentNodeName
                + "\" [label=\"\", shape=box, color=\"green\", style=filled]"));
    }

    public final void beginLoop() {
    }

    public final void start() {
        saveLine("graph G {");
    }

    public final void end(Lbool result) {
        saveLine("}");
    }

    private final String lineTab(final String line) {
        return "\t" + line;
    }

    private final void saveLine(final String line) {
        try {
            out.write(line + '\n');
            if ("}".equals(line)) {
                out.close();
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
}
