package org.sat4j.opt;

import org.sat4j.core.VecInt;
import org.sat4j.specs.ContradictionException;
import org.sat4j.specs.IOptimizationProblem;
import org.sat4j.specs.ISolver;
import org.sat4j.specs.IVecInt;
import org.sat4j.specs.TimeoutException;
import org.sat4j.tools.SolverDecorator;

/**
 * Computes a solution with the smallest number of satisfied literals.
 * 
 * @author leberre
 */
public class MinOneDecorator extends SolverDecorator implements
        IOptimizationProblem {

    /**
     * 
     */
    private static final long serialVersionUID = 1L;

    private int[] prevmodel;

    public MinOneDecorator(ISolver solver) {
        super(solver);
    }

    public boolean admitABetterSolution() throws TimeoutException {
        boolean result = isSatisfiable();
        if (result) {
            prevmodel = super.model();
        }
        return result;
    }

    public boolean hasNoObjectiveFunction() {
        return false;
    }

    public boolean nonOptimalMeansSatisfiable() {
        return true;
    }

    private int counter;

    public Number calculateObjective() {
        counter = 0;
        for (int p : prevmodel) {
            if (p > 0) {
                counter++;
            }
        }
        return counter;
    }

    private final IVecInt literals = new VecInt();

    public void discard() throws ContradictionException {
        if (literals.isEmpty()) {
            for (int i = 1; i <= nVars(); i++) {
                literals.push(i);
            }
        }
        addAtMost(literals, counter - 1);
    }

    @Override
    public int[] model() {
        return prevmodel;
    }

    @Override
    public void reset() {
        literals.clear();
        super.reset();
    }

}
