/*
 * Created on 23 mars 2006
 * 
 * To change the template for this generated file go to
 * Window>Preferences>Java>Code Generation>Code and Comments
 */
package org.sat4j.opt;

import java.math.BigInteger;

import org.sat4j.core.Vec;
import org.sat4j.core.VecInt;
import org.sat4j.specs.ContradictionException;
import org.sat4j.specs.IConstr;
import org.sat4j.specs.IOptimizationProblem;
import org.sat4j.specs.ISolver;
import org.sat4j.specs.IVec;
import org.sat4j.specs.IVecInt;
import org.sat4j.specs.TimeoutException;
import org.sat4j.tools.SolverDecorator;

public class WeightedMaxSatDecorator extends SolverDecorator implements
		IOptimizationProblem {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	private int nborigvars;

	private int nbexpectedclauses;

	private int nbnewvar;

	private final IVec<BigInteger> coefs = new Vec<BigInteger>();

	private int[] prevfullmodel;

	public WeightedMaxSatDecorator(ISolver solver) {
		super(solver);
	}

	@Override
	public int[] model() {
		prevfullmodel = super.model();
		int[] shortmodel = new int[nborigvars];
		for (int i = 0; i < nborigvars; i++) {
			shortmodel[i] = prevfullmodel[i];
		}
		return shortmodel;
	}

	@Override
	public int newVar(int howmany) {
		nborigvars = super.newVar(howmany);
		return nborigvars;
	}

	@Override
	public void setExpectedNumberOfClauses(int nb) {
		nbexpectedclauses = nb;
		super.setExpectedNumberOfClauses(nb);
		super.newVar(nborigvars + nbexpectedclauses);
	}

	@Override
	public IConstr addClause(IVecInt literals) throws ContradictionException {

		coefs.push(BigInteger.valueOf(literals.get(0)));
		literals.set(0, nborigvars + ++nbnewvar);
		return super.addClause(literals);
	}

	@Override
	public void reset() {
		nbnewvar = 0;
		coefs.clear();
        vec.clear();
		super.reset();
	}

	public boolean admitABetterSolution() throws TimeoutException {
		boolean result = super.isSatisfiable();
		if (result)
			prevfullmodel = super.model();
		return result;
	}

	public boolean hasNoObjectiveFunction() {
		return false;
	}

	public boolean nonOptimalMeansSatisfiable() {
		return false;
	}

	private BigInteger counter;

	public Number calculateObjective() {
		counter = BigInteger.ZERO;
		for (int q : prevfullmodel) {
			if (q > nborigvars) {
				counter = counter.add(coefs.get(q - nborigvars - 1));
			}
		}
		return counter;
	}

	private final IVecInt vec = new VecInt();

	public void discard() throws ContradictionException {
		if (vec.isEmpty()) {
			for (int i = nborigvars + 1; i <= nVars(); i++) {
				vec.push(i);
			}
		}
		super.addPseudoBoolean(vec, coefs, false, counter
				.subtract(BigInteger.ONE));
	}

}
