/*
 * Created on 3 juin 2004
 * 
 * To change the template for this generated file go to Window - Preferences -
 * Java - Code Generation - Code and Comments
 */
package org.sat4j.reader;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.net.URL;

import org.sat4j.specs.ContradictionException;
import org.sat4j.specs.IProblem;
import org.sat4j.specs.ISolver;

/**
 * An reader having the responsability to choose the right reader according to
 * the input.
 * 
 * @author leberre
 */
public class InstanceReader extends Reader {

    // private final DimacsReader dimacs;
    private LecteurDimacs dimacs;

    private GoodOPBReader opb;

    private ExtendedDimacsReader edimacs;

    private CSPReader csp;

    private CSPReader csp2;

    private CSPReader csp3;

    private Reader reader = null;

    private final ISolver solver;

    public InstanceReader(ISolver solver) {
        // dimacs = new DimacsReader(solver);
        this.solver = solver;
    }

    private Reader getDefaultSATReader() {
        if (dimacs == null) {
            dimacs = new LecteurDimacs(solver);
        }
        return dimacs;
    }

    private Reader getDefaultOPBReader() {
        if (opb == null) {
            opb = new GoodOPBReader(solver);
        }
        return opb;
    }

    private Reader getDefaultExtendedDimacsReader() {
        if (edimacs == null) {
            edimacs = new ExtendedDimacsReader(solver);
        }
        return edimacs;
    }

    private Reader getCSPReader1() {
        if (csp == null) {
            csp = new CSPReader(solver);
        }
        return csp;
    }

    private Reader getCSPReader2() {
        if (csp2 == null) {
            csp2 = new CSPSupportReader(solver);
        }
        return csp2;
    }

    private Reader getCSPReader3() {
        if (csp3 == null) {
            csp3 = new CSPExtSupportReader(solver);
        }
        return csp3;
    }

    @Override
    public IProblem parseInstance(String filename)
            throws FileNotFoundException, ParseFormatException, IOException,
            ContradictionException {
        String fname;
        boolean isHttp = false;
        String tempFileName = "";
        String prefix = "";
        
        if (filename.startsWith("http://")) {
        	isHttp = true;
        	tempFileName = filename;
        	filename = filename.substring(filename.lastIndexOf('/'), filename.length() - 1);
        }
        
        if (filename.indexOf(':') != -1) {
        	
        	String[] parts = filename.split(":");
        	filename = parts[1];
        	prefix = parts[0].toUpperCase();
        	
        }
        
        if (filename.endsWith(".gz")) {
            fname = filename.substring(0, filename.lastIndexOf('.'));
        } else {
            fname = filename;
        }
        if ("CSP".equals(prefix)) {
            reader = getCSPReader1();
        } else if ("CSP3".equals(prefix)) {
            reader = getCSPReader3();
        } else if (fname.endsWith(".txt") || "CSP2".equals(prefix)) {
            reader = getCSPReader2();
        } else if (fname.endsWith(".opb") || "PB".equals(prefix)) {
            reader = getDefaultOPBReader();
        } else if (fname.endsWith(".edimacs") || fname.endsWith(".ncnf")
                || "EDIMACS".equals(prefix)) {
            reader = getDefaultExtendedDimacsReader();
        } else {
            reader = getDefaultSATReader();
        }
        
        if (isHttp) {
        	return reader.parseInstance((new URL(tempFileName)).openStream());
        }
        return reader.parseInstance(filename);
    }

    @Override
    public String decode(int[] model) {
        return reader.decode(model);
    }


	@Override
    public IProblem parseInstance(java.io.Reader in) throws ParseFormatException, ContradictionException, IOException {
		throw new UnsupportedOperationException();
	}
}
