/*
 * Created on 15 sept. 2004
 *
 * To change the template for this generated file go to
 * Window - Preferences - Java - Code Generation - Code and Comments
 */
package org.sat4j.reader;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.URL;
import java.util.zip.GZIPInputStream;

import org.sat4j.specs.ContradictionException;
import org.sat4j.specs.IProblem;

/**
 * A reader is responsible to feed an ISolver from a text file and to convert
 * the model found by the solver to a textual representation.
 * 
 * @author leberre
 */
public abstract class Reader {

	public IProblem parseInstance(final String filename)
			throws FileNotFoundException, ParseFormatException, IOException,
			ContradictionException {
		InputStream in;
		if (filename.startsWith("http://")) {
			in = (new URL(filename)).openStream();
		} else {
			in = new FileInputStream(filename);
		}
		if (filename.endsWith(".gz")) {
			in = new GZIPInputStream(in);
		}
		return parseInstance(in);
	}

	public IProblem parseInstance(final InputStream in)
			throws ParseFormatException, ContradictionException, IOException {
		return parseInstance(new InputStreamReader(in));
	}

	public abstract IProblem parseInstance(final java.io.Reader in)
			throws ParseFormatException, ContradictionException, IOException;

	/**
	 * Produce a model using the reader format.
	 * 
	 * @param model
	 *            a model using the Dimacs format.
	 * @return a human readable view of the model.
	 */
	public abstract String decode(int[] model);
}
