package org.sat4j.reader.csp;

import org.sat4j.core.VecInt;
import org.sat4j.specs.ContradictionException;
import org.sat4j.specs.ISolver;
import org.sat4j.specs.IVecInt;


public class SupportsDirectEncoding implements Relation {

    protected final int[] domains;

    protected int[][] tuples;

    private IVecInt clause;
    private int lastmatch;
    
    public SupportsDirectEncoding(int[] domains, int nbtuples) {
        this.domains = domains;
        tuples = new int[nbtuples][];
    }

    public void addTuple(int index, int[] tuple) {
        tuples[index] = tuple;
    }

    public void toClause(ISolver solver, Var[] vars)
            throws ContradictionException {
        // need to find all the tuples that are not expressed here.
        clause = new VecInt();
        int[] tuple = new int[vars.length];
        lastmatch=-1;
        find(tuple, 0, vars, solver);
    }

    private void find(int[] tuple, int n, Var[] vars, ISolver solver)
            throws ContradictionException {
        if (n == vars.length) {
            if (notPresent(tuple)) {
                clause.clear();
                for (int j = 0; j < vars.length; j++) {
                    clause.push(-vars[j].translate(tuple[j]));
                }
                // System.err.println("Adding clause:" + clause);
                solver.addClause(clause);
            }
        } else {
            int[] domain = vars[n].domain();
            for (int i = 0; i < domain.length; i++) {
                tuple[n] = domain[i];
                find(tuple, n + 1, vars, solver);
            }

        }

    }

    private boolean notPresent(int[] tuple) {
        // System.out.println("Checking:" + Arrays.asList(tuple));
        // find the first tuple begining with the same
        // initial number
        int i = lastmatch+1;
        int j = 0;
        while (i<tuples.length&&j < tuple.length) {
            if (tuples[i][j] < tuple[j]) {
                i++;
                j = 0;
                continue;
            }
            if (tuples[i][j] > tuple[j])
                return true;
            j++;
        }
        if (j==tuple.length) {
            lastmatch=i;
            return false;
        }
        return true;
    }

    public int arity() {
        return domains.length;
    }
}