package org.sat4j.reader.csp;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.sat4j.core.VecInt;
import org.sat4j.specs.ContradictionException;
import org.sat4j.specs.ISolver;
import org.sat4j.specs.IVecInt;

public class SupportsGeneralizedACEncoding extends SupportsDirectEncoding {

    public SupportsGeneralizedACEncoding(int[] domains, int nbtuples) {
        super(domains, nbtuples);
    }

    @Override
    public void toClause(ISolver solver, Var[] vars)
            throws ContradictionException {
        Map<Set<Integer>, IVecInt> supports = new HashMap<Set<Integer>, IVecInt>();
        int[] acc = new int[vars.length];
        fill(0, vars, acc, supports);

        for (int[] tuple : tuples) {
            for (int i = 0; i < tuple.length; i++) {
                Set<Integer> set = new HashSet<Integer>();
                for (int j = 0; j < tuple.length; j++) {
                    if (i != j) {
                        set.add(vars[j].translate(tuple[j]));
                    }
                }
                IVecInt support = supports.get(set);
                assert support!=null;
                support.push(vars[i].translate(tuple[i]));
            }
        }

        IVecInt clause = new VecInt();
        for (Map.Entry<Set<Integer>, IVecInt> entry : supports.entrySet()) {
            clause.clear();
            // System.out.println(" "+entry.getKey()+" -> "+entry.getValue());
            for (int p : entry.getKey())
                clause.push(-p);
            for (int i : entry.getValue())
                clause.push(i);
            solver.addClause(clause);
        }
    }

    private void fill(int n, Var[] vars, int[] acc,
            Map<Set<Integer>, IVecInt> supports) {
        if (n == vars.length) {
            for (int j = 0; j < acc.length; j++) {
                Set<Integer> set = new HashSet<Integer>();
                for (int i = 0; i < acc.length; i++)
                    if (i != j)
                        set.add(vars[i].translate(acc[i]));
                supports.put(set, new VecInt());
            }
        } else
            for (int i : vars[n].domain()) {
                acc[n] = i;
                fill(n + 1, vars, acc, supports);
            }

    }
}
